@@ -24,9 +24,12 @@ directories = {
     'postprocessor': '',
     'schema': '',
     'tester': '',
+    'test_schema': '',
     'trainer': '',
     'server': '',
 }
+
+
 class AliasedGroup(click.Group):
     def get_command(self, ctx, cmd_name):
         try:
@@ -35,6 +38,7 @@ class AliasedGroup(click.Group):
             pass
         return super().get_command(ctx, cmd_name)
 
+
 @click.group(cls=AliasedGroup)
 def cli():
     """
@@ -60,6 +64,10 @@ def cli():
 
     dsf-cli g predictable my-new-predictable
 
+    create tester files
+
+    dsf-cli create-tester-files
+
     run server
 
     dsf-cli run-server
@@ -68,6 +76,14 @@ def cli():
 
     dsf-cli create-deploy-files
 
+    create cloud eval files
+
+    dsf-cli create-cloud-eval-files
+
+    run evaluation on a csv input
+
+    dsf-cli evaluate input-csv-file batch-size
+
     """
 # @click.option("--type", prompt="type", help="type of component")
 # @click.option("--project_name", prompt="project_name", help="project_name")
@@ -75,17 +91,18 @@ def cli():
 
 
 @cli.command()
-@click.argument('type')
+@click.argument('gen_type')
 @click.argument('project_name')
-def generate(type, project_name):
+def generate(gen_type, project_name):
     """List all cataloged APIs."""
     try:
-        f = globals()["generate_%s" % type]
+        f = globals()["generate_%s" % gen_type]
     except Exception as e:
-        click.echo('type ' + type + ' not found')
+        click.echo('type ' + gen_type + ' not found')
         return
     f(project_name)
 
+
 @cli.command()
 def run_server():
     currentPipelineFolder = os.path.abspath(os.getcwd())
@@ -94,31 +111,62 @@ def run_server():
     folder = currentPipelineFolder + '/server/main.py'
     subprocess.call('python ' + folder, shell=True)
 
+
 @cli.command()
 def create_deploy_files():
     copy_deploy_files('')
 
+
+@cli.command()
+def create_cloud_eval_files():
+    currentPipelineFolder = os.path.abspath(os.getcwd())
+    copy_cloud_eval_files(currentPipelineFolder)
+
+
+@cli.command()
+def create_tester_files():
+    create_tester_env()
+
+
+@cli.command()
+@click.argument('csv_file_path')
+@click.argument('batch_size')
+def evaluate(csv_file_path, batch_size):
+    if os.path.isfile(csv_file_path):
+        current_pipeline_folder = os.path.abspath(os.getcwd())
+        os.environ["PYTHONPATH"] = current_pipeline_folder
+        folder = current_pipeline_folder + '/tester/general_tester.py'
+        subprocess.call('python ' + folder + ' ' + csv_file_path + ' ' + batch_size, shell=True)
+    else:
+        click.echo('file: ' + csv_file_path + ' not found')
+
+
 ALIASES = {
     "g": generate
 }
 
+
 def generate_project(project_name):
     # project_name = clean_name(project_name)
-    click.echo('project generated ' + project_name)
+    click.echo('Generating project: ' + project_name)
     create_project(project_name)
 
+
 def generate_forcer(fileName):
     fileName = clean_name(fileName)
     create_exist_pipeline_file('forcer', fileName)
 
+
 def generate_predictable(fileName):
     fileName = clean_name(fileName)
     create_exist_pipeline_file('predictable', fileName)
 
+
 def clean_name(name):
     name = name.replace('-', '_')
     return name
 
+
 def create_folders(project_name):
     global directories
     directories['main'] = project_name
@@ -138,9 +186,10 @@ def create_folders(project_name):
     create_main_folders('postprocessor', 'pipeline')
     create_main_folders('schema', 'pipeline')
     create_main_folders('tester', 'main')
+    create_main_folders('test_schema', 'tester')
     create_main_folders('trainer', 'main')
     create_main_folders('server', 'main')
-    create_main_folders('test_scheme', 'server')
+
 
 def create_project(project_name):
     create_folders(project_name)
@@ -152,16 +201,17 @@ def create_project(project_name):
     create_pipeline_file(project_name, directories['postprocessor'], 'postprocess')
     create_pipeline_file(project_name, directories['predictables'], 'predictable')
     create_pipeline_file(project_name, directories['pipeline'], 'pipeline')
-    create_pipeline_file(project_name, directories['main'], 'test_pipeline', False)
+    create_pipeline_file(project_name, directories['main'], 'pipeline_test', False)
 
-    create_scheme_file(project_name, directories['schema'], 'inputs')
-    create_scheme_file(project_name, directories['schema'], 'outputs')
-    create_scheme_file(project_name, directories['test_scheme'], 'test_input', False)
-    create_scheme_file(project_name, directories['test_scheme'], 'test_output', False)
-    create_scheme_file(project_name, directories['test_scheme'], '__init__', False)
+    create_schema_file(project_name, directories['schema'], 'inputs')
+    create_schema_file(project_name, directories['schema'], 'outputs')
+    create_schema_file(project_name, directories['schema'], '__init__', False)
+
+    create_tester_file(project_name, directories['tester'], 'general_tester', False)
+    create_tester_file(project_name, directories['tester'], 'evaluator', False)
+    create_schema_file(project_name, directories['test_schema'], 'test_input', False)
+    create_schema_file(project_name, directories['test_schema'], 'test_output', False)
 
-    # create_tester_file(project_name, directories['tester'], 'tester')
-    # create_tester_file(project_name, directories['tester'], 'reporter')
     create_server_file(project_name, directories['server'], 'main', False)
     create_server_file(project_name, directories['server'], 'test_server_post', False)
     create_server_file(project_name, directories['server'], 'pool', False)
@@ -194,21 +244,26 @@ def create_main_folders(targetDir, baseDir):
     if not os.path.exists(directories[targetDir]):
         os.mkdir(directories[targetDir])
 
+
 def create_pipeline_file(project_name, folder, pipelineType, createInitFile=True):
     data = read_template_file(pipelineType)
     replace_in_template_and_create_file(project_name, folder, pipelineType, data, createInitFile)
 
-def create_scheme_file(project_name, folder, pipelineType, createInitFile=True):
-    data = read_template_file('scheme/' + pipelineType)
+
+def create_schema_file(project_name, folder, pipelineType, createInitFile=True):
+    data = read_template_file('schema/' + pipelineType)
     replace_in_template_and_create_file(project_name, folder, pipelineType, data, createInitFile)
 
+
 def create_server_file(project_name, folder, pipelineType, createInitFile=True):
     data = read_template_file('tester/server/' + pipelineType)
     replace_in_template_and_create_file(project_name, folder, pipelineType, data, createInitFile)
 
-def create_tester_file(project_name, folder, pipelineType, createInitFile=True):
-    data = read_template_file('tester/' + pipelineType)
-    replace_in_template_and_create_file(project_name, folder, pipelineType, data, createInitFile)
+
+def create_tester_file(project_name, folder, test_file_type, createInitFile=True):
+    data = read_template_file('tester/' + test_file_type)
+    replace_in_template_and_create_file(project_name, folder, test_file_type, data, createInitFile)
+
 
 def create_exist_pipeline_file(type, fileName):
     pipelineType = type + 's'
@@ -234,21 +289,68 @@ def create_exist_pipeline_file(type, fileName):
     pass
 
 
-def read_template_file(pipelineType):
-    with open(os.path.join(__location__, 'cli/' + pipelineType + '_template.py'), 'r') as file:
+def create_tester_env():
+    """
+    This function will create a testing environment, if one does not already exist in the project.
+    It will create the testing, evaluation, and relevant schema files.
+    """
+    project_name = os.getcwd().split('/')[-1]
+    dst_folder = 'tester/'
+    schema_dst_folder = 'tester/test_schema/'
+    tester_file_list = ["tester/general_tester",
+                        "tester/evaluator",
+                        "schema/test_input",
+                        "schema/test_output"]
+
+    if not os.path.exists(dst_folder):
+        os.mkdir(dst_folder)
+        os.mkdir(schema_dst_folder)
+
+        for filename_with_path in tester_file_list:
+            file_only = filename_with_path.split('/')[-1]
+            if file_only.startswith('test'):
+                dst_folder = schema_dst_folder
+
+            data = read_template_file(filename_with_path)
+
+            clean_project_name = clean_existing_project_name(project_name)
+            filename_no_underscore = to_capitalize_no_underscore(file_only)
+
+            class_name = clean_project_name + filename_no_underscore
+
+            data = data.replace('generatedClass', class_name)
+            data = data.replace('generatedProjectName', clean_project_name)
+
+            new_file = dst_folder + "/" + file_only + ".py"
+            create_file(new_file, data)
+
+            # Test schema does not use generated class name for now, so this code should be modified before uncommenting
+            # current_init_file = dst_folder + "/" + "__init__.py"
+            # new_init_export = "from " + dst_folder.replace('/', '.') + "." + file_only + " import " + class_name
+            # create_init_file(current_init_file, new_init_export)
+    else:
+        print('Tester folder Already exists. If you wish to recreate, please erase folder and retry')
+    pass
+
+
+def read_template_file(filename):
+    with open(os.path.join(__location__, 'cli/' + filename + '_template.py'), 'r') as file:
         data = file.read()
         return data
 
+
 def read_file(file_path):
     with open(file_path, 'r') as file:
         data = file.read()
         return data
 
+
 def write_to_file(file_path, data):
     with open(file_path, 'w') as file:
         file.write(data)
         file.close()
 
+
 def replace_in_template_and_create_file(project_name, folder, pipelineType, data, createInitFile):
     pipeline_type_no_underscore = to_capitalize_no_underscore(pipelineType)
     project_name_no_underscore = to_capitalize_no_underscore(project_name)
@@ -275,14 +377,18 @@ def replace_in_template_and_create_file(project_name, folder, pipelineType, data
         new_init_export = "from " + currentDirNoMainDir + '.' + pipelineType + " import " + className
         create_init_file(new_init_file, new_init_export)
 
+
 def create_predictable_file(fileName):
     pass
 
+
 def create_file(new_file_path, data):
     if not os.path.exists(new_file_path):
         f = open(new_file_path, "w")
         f.write(data)
         f.close()
+
+
 def create_init_file(init_path, init_export):
     if not os.path.exists(init_path):
         f = open(init_path, "w")
@@ -297,6 +403,7 @@ def create_init_file(init_path, init_export):
             f.write(init_export)
             f.close()
 
+
 def inject_to_pipeline(fileName, className, new_init_export):
     file_path = 'pipeline/pipeline.py'
     if os.path.exists(file_path):
@@ -380,6 +487,7 @@ def inject_to_pipeline(fileName, className, new_init_export):
         if data_changed:
             write_to_file(file_path, data)
 
+
 def create_project_config_yaml():
     with open(os.path.join(__location__, 'cli/config.yaml'), 'r') as file:
         data = file.read()
@@ -390,6 +498,7 @@ def create_project_config_yaml():
             f.write(data)
             f.close()
 
+
 def create_project_config_json():
     with open(os.path.join(__location__, 'cli/config.json'), 'r') as file:
         data = file.read()
@@ -400,6 +509,7 @@ def create_project_config_json():
             f.write(data)
             f.close()
 
+
 def create_project_gitignore():
     with open(os.path.join(__location__, 'cli/.gitignore'), 'r') as file:
         data = file.read()
@@ -409,6 +519,7 @@ def create_project_gitignore():
             f.write(data)
             f.close()
 
+
 def create_server_config_json():
     with open(os.path.join(__location__, 'cli/cors_allowed_origins.json'), 'r') as file:
         data = file.read()
@@ -419,9 +530,11 @@ def create_server_config_json():
             f.write(data)
             f.close()
 
+
 def change_to_project_dir():
     os.chdir(directories['main'])
 
+
 def run_dvc_init():
     dir_path = os.getcwd()
     if not os.path.isdir(dir_path + '/.dvc'):
@@ -430,6 +543,7 @@ def run_dvc_init():
             command = './' + command
         subprocess.call(command, shell=True)
 
+
 def run_git_init():
     dir_path = os.getcwd()
     if not os.path.isdir(dir_path + '/.git'):
@@ -438,6 +552,7 @@ def run_git_init():
             command = './' + command
         subprocess.call(command, shell=True)
 
+
 def copy_deploy_files(main_dir):
     currentPipelineFolder = os.path.basename(os.getcwd())
     if main_dir:
@@ -475,8 +590,72 @@ def copy_deploy_files(main_dir):
     #     else:
     #         shutil.copy2(s, d)
 
+
+def copy_cloud_eval_files(main_dir):
+    currentPipelineFolder = os.path.basename(os.getcwd())
+    if main_dir:
+        currentPipelineFolder = main_dir
+    service_folder = str(currentPipelineFolder)
+    service_name = service_folder.split('/')[-1]
+    dir = os.path.join(__location__, 'cli/tester/cloud_eval/')
+    listOfFiles = list()
+    conflicted_files = []
+    for (dirpath, dirname, filenames) in os.walk(dir):
+        if "__pycache__" in dirpath:
+            continue
+        dirpath = dirpath.replace(dir, '')
+        listOfFiles += [os.path.join(dirpath, file) for file in filenames]
+        for file_path in listOfFiles:
+            framework_source_file = os.path.join(dir, file_path)
+            with open(framework_source_file, 'r', encoding="utf8") as file:
+                try:
+                    data = file.read()
+                    data = data.replace('{name-your-service}', service_name)
+                    data = data.replace('{name-your-artifacts}', service_name)
+                    dirToCreate = ''
+                    if main_dir:
+                        dirToCreate = main_dir + '/cloud_eval/'
+                    if not os.path.exists(dirToCreate):
+                        os.makedirs(dirToCreate)
+                    if "#cli_file_dest=[" in data:
+                        start_ix = data.index("#cli_file_dest=[") + len("#cli_file_dest=[")
+                        end_ix = data.index("]", start_ix)
+                        destination_dir = data[start_ix:end_ix]
+                        data = data[:data.index("#cli_file_dest=[")] + data[end_ix+1:]
+                        dirToCreate = main_dir + '/' + destination_dir
+
+                    if not os.path.exists(dirToCreate + file_path):
+                        f = open(dirToCreate + file_path, "w")
+                        f.write(data)
+                        f.close()
+                        # Make sure that .sh files can be executed
+                        if (dirToCreate + file_path).endswith('.sh'):
+                            os.chmod(dirToCreate + file_path, 0o744)
+                    else:
+                        with open(dirToCreate + file_path, "r") as f:
+                            existing_data = f.read()
+                        if existing_data != data:
+                            conflicted_file = f"{dirToCreate + file_path}"
+                            conflicted_files.append((conflicted_file, str(framework_source_file)))
+
+                except Exception as ex:
+                    pass
+
+    if conflicted_files:
+        print("The following files already exists, so not overwriting them. "
+              "However, notice it is different from the framework version in")
+        for conflicted_file in conflicted_files:
+            print(f"Existing: [{conflicted_file[0]}] \t\t Framework source file [{conflicted_file[1]}]")
+
+
 def to_capitalize_no_underscore(text):
     return ''.join(elem.capitalize() for elem in text.split('_'))
 
+
+def clean_existing_project_name(text):
+    no_underscore_text = to_capitalize_no_underscore(text)
+    return ''.join(elem.capitalize() for elem in no_underscore_text.split('-'))
+
+
 if __name__ == '__main__':
     cli(prog_name='cli')
