@@ -26,13 +26,16 @@ from starlette.requests import Request
 from pipeline.pipeline import generatedProjectNamePipeline
 from pipeline.schema.inputs import generatedProjectNameInputs
 from pipeline.schema.outputs import generatedProjectNameOutputs
-from server.test_scheme.test_input import TestInputRequest
-from server.test_scheme.test_output import TestOutputResponse
+from tester.test_schema.test_input import TestInputRequest
+from tester.test_schema.test_output import TestOutputResponse
+from tester.general_tester import generatedProjectNameGeneralTester
+
 
 async def get_token_header(x_token: str = Header(...)):
     if x_token != "fake-super-secret-token": # todo: please replace with realistic token
         raise HTTPException(status_code=400, detail="X-Token header invalid")
 
+
 def load_allowed_cors_origins():
     allowed_origins = []
     cors_origin_list_path = 'server/cors_allowed_origins.json'
@@ -63,95 +66,77 @@ pipeline = generatedProjectNamePipeline()
 # pipelines = [generatedProjectNamePipeline() for i in range(3)]
 # app.pipelines = InstancePool(pipelines)
 
+
 @app.post('/predict', dependencies=[Depends(get_token_header)])
 def predict(body: generatedProjectNameInputs, request: Request) -> List[generatedProjectNameOutputs]:
     '''
     predict api
     '''
     data = body.dict()
-    logger.info({"message": "predict invoked", "data": json.dumps(data)})
-    print('data', data)
+    # logger.info({"message": "predict invoked", "data": json.dumps(data)})
+    # print('data', data)
 
     # call model here
     try:
-        output: List[generatedProjectNameOutputs] = pipeline.execute(**data)
+        output: generatedProjectNameOutputs = pipeline.execute(**data)
+        logger.info("INFO predict invoked", extra={"input": data, "output": output.dict()})
     except Exception as ex:
         logger.exception(msg=data, exc_info=True)
         output = {'error': {'request': ex}}
     # call model here
     return output
 
+
 @app.post('/parse', dependencies=[Depends(get_token_header)])
 def parse(body: generatedProjectNameInputs, request: Request) -> List[generatedProjectNameOutputs]:
     '''
     parse api
     '''
     data = body.dict()
-    logger.info({"message": "parse invoked", "data": json.dumps(data)})
-    print('data', data)
+    # logger.info({"message": "parse invoked", "data": json.dumps(data)})
+    # print('data', data)
 
     # call model here
     try:
-        output: List[generatedProjectNameOutputs] = pipeline.execute(**data)
+        output: generatedProjectNameOutputs = pipeline.execute(**data)
+        logger.info("INFO parse invoked", extra={"input": data, "output": output.dict()})
     except Exception as ex:
         logger.exception(msg=data, exc_info=True)
         output = {'error': {'request': data}}
     # call model here
     return output
 
+
+# Fetch the proper tester for the project
+model_tester = generatedProjectNameGeneralTester()
+
+
 @app.post('/test', dependencies=[Depends(get_token_header)])
-def test(body: TestInputRequest) -> List[TestOutputResponse]:
-    '''
-    test api
-    '''
-    body = body.dict()
-    logger.info({"message": "test invoked", "test": json.dumps(body)})
-    print('body', body)
+def test(input_request: TestInputRequest) -> List[TestOutputResponse]:
+    """
+    This is the main endpoint for testing. Every project starts with a mock response service, in order to enable
+    DSP integration, but as soon as a working testing functions are ready, please replace the mock service with the
+    real one.
 
-    # call tester here
+    Parameters:
+        input_request: Request as received from the DSP
+    Returns:
+        A list of responses, to be sent back to the DSP
+    """
+    response = model_tester.create_mock_response(input_request)
+
+    """    
+    model_tester.save_meta_data(input_request)
     response = []
-    rows = body['rows']
-    truth_dataset_id = body['truth_dataset_id']
-    model_type_id = body['model_type_id']
-    for row in rows:
-        data = json.loads(row['data'])
-        id = row['id']
-        raw_id = row['raw_id'] or -1
-        segments = data['segments'] if 'segments' in data else data
-        pred = data['segments'] if 'segments' in data else data
-        text = ''
-        if 'signature' in data:
-            text = data['signature']
-        elif 'email' in data:
-            text = data['email']
-        elif 'text' in data:
-            text = data['text']
-        elif 'name' in data:
-            text = data['name']
-        response.append(
-            TestOutputResponse(
-                **{
-                    'truth_id': id,
-                    'truth_dataset_id': truth_dataset_id,
-                    'model_type_id': model_type_id,
-                    'raw_id': raw_id,
-                    'pred': json.dumps(pred),
-                    'target': json.dumps(segments),
-                    'text': text,
-                    'name_first_fn': 0,
-                    'name_first_fp': 0,
-                    'name_first_pred': "Marty",
-                    'name_first_prob': -1,
-                    'name_first_target': "Marty",
-                    'name_first_tn': 0,
-                    'name_first_tp': 1,
-                }
-            )
-        )
-    # call tester here
-
+    # Until this model will support batches, work sequentially. Once it does, pass the full list of rows.
+    for request_row in input_request.rows:
+        input_list = [request_row]
+        tester_output = model_tester.test_batch(input_list)
+        response.append(tester_output[0])
+    """
     return response
 
+
 @app.get("/livenessprobe")
 def liveness_probe():
     return {"alive": True}
@@ -161,6 +146,7 @@ def liveness_probe():
 def readiness_probe():
     return {"ready": True}
 
+
 if __name__ == '__main__':
     uvicorn.run(app, host='0.0.0.0', port=8080)
 
