@@ -6,6 +6,7 @@ from dsframework.base.common.regex_handler import RegexHandler
 
 regex = RegexHandler
 
+
 def flatten(d,sep="_"):
     import collections
 
@@ -21,6 +22,7 @@ def flatten(d,sep="_"):
     recurse(d)
     return obj
 
+
 def get_from_dict(key, container:dict, default='', delete=False):
     if not container or type(container) is not dict:
         return default
@@ -31,32 +33,42 @@ def get_from_dict(key, container:dict, default='', delete=False):
         return val
     return default
 
-def get_date_time(short=False):
+
+def get_date_time(short=False, no_space=False):
     now = datetime.now()
-    form = '%d/%m/%Y %H:%M:%S' if not short else '%d_%m_%Y'
+    form = '%d/%m/%Y %H:%M:%S'
+    if short:
+        form = '%d_%m_%Y'
+    elif no_space:
+        form = "%Y%m%d-%H%M%S"
     dt_string = now.strftime(form)
     return dt_string
 
+
 def load_pickle(path):
     pkl = None
     with open(path, 'rb') as fid:
-        pkl =  pickle.load(fid)
+        pkl = pickle.load(fid)
     return pkl
 
+
 def load_json(path):
     jsn = None
     with open(path) as fid:
         jsn =  json.load(fid)
     return jsn 
 
+
 def save_pickle(obj, path):
     with open(path, 'wb') as fid:
         pickle.dump(obj, fid)
 
+
 def save_json(obj, path):
     with open(path, 'w') as f:
         json.dump(f, obj)
 
+
 def remove_empty_leafs(d):
     if not isinstance(d, (dict, list)):
         return d
@@ -64,6 +76,7 @@ def remove_empty_leafs(d):
         return [v for v in (remove_empty_leafs(v) for v in d) if v]
     return {k: v for k, v in ((k, remove_empty_leafs(v)) for k, v in d.items()) if v}
 
+
 def flatten_list(l):
     def get_el(el):
         if type(el) is not list:
@@ -73,12 +86,14 @@ def flatten_list(l):
     flatten = lambda l: [item for sublist in l for item in sublist]
     return flatten(l2)
 
+
 def is_list_of_list(l):
     for el in l:
         if type(el) is list:
             return True
     return False
 
+
 def ngram_splitter(line, ngram=3,all_grams=False):
     tokens       = line.split(" ")
     loop_index   = 1
@@ -100,13 +115,15 @@ def ngram_splitter(line, ngram=3,all_grams=False):
             ngram_window = ngram_window[1:]
         loop_index +=1 
     return result
-    
+
+
 def remove_duplicates_lines(lines = []):
     ls = lines 
     lookup = set()  # a temporary lookup set
     ls = [x for x in ls if x not in lookup and lookup.add(x) is None]
     return ls 
-    
+
+
 def get_html_block_elements_list():
     return ['<address', '<article', '<aside', '<blockquote', '<canvas', '<dd', '<div',
             '<dl',
@@ -116,6 +133,7 @@ def get_html_block_elements_list():
             '<noscript', '<ol', '<p', '<pre', '<section', '<table', '<tfoot', '<ul',
             '<video']
 
+
 def get_html_inline_elements_list():
     return ['<a', '<abbr', '<acronym', '<b', '<bdo', '<big', '<br', '<button', '<cite',
             '<code', '<dfn', '<em',
@@ -123,3 +141,17 @@ def get_html_inline_elements_list():
             '<q', '<samp', '<script',
             '<select', '<small', '<span', '<strong', '<sub', '<sup', '<textarea',
             '<time', '<tt', '<var']
+
+
+def split_dataframe(df, batch_size=3):
+    """
+    Helper function that can chunk a dataframe according to a given chunk size
+    :param df: Dataframe to divide
+    :param batch_size: Batch size
+    :return: A list of chunks, each the size of the desired batch size
+    """
+    chunks = list()
+    num_chunks = len(df) // batch_size + 1
+    for i in range(num_chunks):
+        chunks.append(df[i * batch_size:(i + 1) * batch_size])
+    return chunks
